ENV_VAR_CONTAINER_GROUP_NAME = 'AZURE_CONTAINER_GROUP_NAME'
ENV_VAR_RESOURCE_GROUP_NAME = 'AZURE_RESOURCE_GROUP_NAME'
ENV_VAR_STORAGE_ACCOUNT_KEY = 'AZURE_STORAGE_ACCOUNT_KEY'
ENV_VAR_SUBSCRIPTION_ID = 'AZURE_SUBSCRIPTION_ID'
