CONNECTION_STATUS_ACTIVE = 'active'
CONNECTION_STATUS_DEPRECATED = 'deprecated'
CONNECTION_STATUS_INACTIVE = 'inactive'

JOB_STATUS_FAILED = 'failed'
JOB_STATUS_PENDING = 'pending'
JOB_STATUS_SUCCEEDED = 'succeeded'
