LOG_LEVEL_DEBUG = 'DEBUG'
LOG_LEVEL_ERROR = 'ERROR'
LOG_LEVEL_EXCEPTION = 'EXCEPTION'
LOG_LEVEL_INFO = 'INFO'

TYPE_ACTIVATE_VERSION = 'ACTIVATE_VERSION'
TYPE_LOG = 'LOG'
TYPE_RECORD = 'RECORD'
TYPE_SAMPLE_DATA = 'SAMPLE_DATA'
TYPE_SCHEMA = 'SCHEMA'
TYPE_STATE = 'STATE'
