"use strict";(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[440],{20440:function(o){o.exports=JSON.parse('{"base":"vs-dark","inherit":true,"rules":[{"background":"0D0D0D","token":""},{"foreground":"473c45","token":"comment"},{"foreground":"805978","token":"string"},{"foreground":"a8885a","token":"constant"},{"foreground":"596380","token":"variable.parameter"},{"foreground":"596380","token":"variable.other"},{"foreground":"728059","token":"keyword - keyword.operator"},{"foreground":"728059","token":"keyword.operator.logical"},{"foreground":"9ebf60","token":"storage"},{"foreground":"6078bf","token":"entity"},{"fontStyle":"italic","token":"entity.other.inherited-class"},{"foreground":"8a4b66","token":"support"},{"foreground":"893062","token":"support.type.exception"},{"background":"5f0047","token":"invalid"},{"background":"371d28","token":"meta.function.section"}],"colors":{"editor.foreground":"#DDE6CF","editor.background":"#0D0D0D","editor.selectionBackground":"#40002F","editor.lineHighlightBackground":"#00000012","editorCursor.foreground":"#7F005D","editorWhitespace.foreground":"#BFBFBF"}}')}}]);