COOKIE_OAUTH_TOKEN = 'oauth_token'
ENDPOINTS_BYPASS_OAUTH_CHECK = []
HEADER_API_KEY = 'X-API-KEY'
HEADER_OAUTH_TOKEN = 'OAUTH-TOKEN'
OAUTH_CLIENT_ID_KEY = 'OAUTH_CLIENT_ID'
URL_PARAMETER_API_KEY = 'api_key'
