# Read data
{% for input_path in input_paths %}
df_{{loop.index}} = read.csv(file="{{input_path}}")
{% endfor %}

{{ global_vars }}

# Define export function
{{ code }}

# Execute export function
export_data({{input_vars_str}})
