"use strict";(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[7951],{57951:function(o){o.exports=JSON.parse('{"base":"vs","inherit":true,"rules":[{"background":"FFFFFF","token":""},{"foreground":"bcc8ba","token":"comment"},{"foreground":"5d90cd","token":"string"},{"foreground":"46a609","token":"constant.numeric"},{"foreground":"39946a","token":"constant.language"},{"foreground":"af956f","token":"keyword"},{"foreground":"af956f","token":"support.constant.property-value"},{"foreground":"af956f","token":"constant.other.color"},{"foreground":"96dc5f","token":"keyword.other.unit"},{"foreground":"484848","token":"keyword.operator"},{"foreground":"c52727","token":"storage"},{"foreground":"858585","token":"entity.other.inherited-class"},{"foreground":"606060","token":"entity.name.tag"},{"foreground":"bf78cc","token":"constant.character.entity"},{"foreground":"bf78cc","token":"support.class.js"},{"foreground":"606060","token":"entity.other.attribute-name"},{"foreground":"c52727","token":"meta.selector.css"},{"foreground":"c52727","token":"entity.name.tag.css"},{"foreground":"c52727","token":"entity.other.attribute-name.id.css"},{"foreground":"c52727","token":"entity.other.attribute-name.class.css"},{"foreground":"484848","token":"meta.property-name.css"},{"foreground":"c52727","token":"support.function"},{"background":"ff002a","token":"invalid"},{"foreground":"c52727","token":"punctuation.section.embedded"},{"foreground":"606060","token":"punctuation.definition.tag"},{"foreground":"bf78cc","token":"constant.other.color.rgb-value.css"},{"foreground":"bf78cc","token":"support.constant.property-value.css"}],"colors":{"editor.foreground":"#000000","editor.background":"#FFFFFF","editor.selectionBackground":"#BDD5FC","editor.lineHighlightBackground":"#FFFBD1","editorCursor.foreground":"#000000","editorWhitespace.foreground":"#BFBFBF"}}')}}]);